/*
Program: eq-gen
File: main.c
Author: Martin Voros
*/

#include <stdio.h>
#include <getopt.h>
#include "main.h"
#include "comb.h"
#include "common.h"

short logic=0;
short sol=0;
unsigned int deg=2;
unsigned long field=2;
unsigned long var=6;
unsigned long num=0;
unsigned long prob=4;
char *output=NULL;
unsigned long counter=0;

int
main(int argc, char* argv[])
{
	FILE * out;
	int ** all=NULL;

	if (argc==1) 
		print_help(stdout, 0);
	else 
		arguments(argc, argv);
	all=gen_comb(deg, var);
	if (output == NULL) {
		out=stdout;
	} else {
		out=xfopen(output, "w");
	}
	if (prob == 0) 
		prob=2;
	
	fprintf(out ,"%% Generating %lu equations with variables [x_0,..,x_%lu]"
			" and with degree %lu over finite ring F_%lu: \n"
			,num, var-1, deg, field);
	gen_eq(out, num, all, counter, field, prob, logic, sol, var);
	
	fclose(out);
	return 0;
}

/* 
 * parse arguments 
 */
void
arguments(int argc, char* argv[])
{
	int next;
	const char* const short_options = "Vhlf:n:o:d:v:p:s";
	do {
	next = getopt(argc, argv, short_options);
	switch (next) {
		case 'h' : print_help(stdout, 0); break;
		case 'V' : 	fprintf(stdout,"Eq-gen version %s\n",VERSION); 
				exit(0); 
				break;
		case 'f' : field=atol(optarg); break;
		case 's' : sol=1; break;
		case 'o' : output=optarg; break;
		case 'n' : num=atol(optarg); break;
		case 'v' : var=atol(optarg); break;
		case 'p' : prob=atol(optarg); break;
		case 'd' : deg=atoi(optarg); break;
		case 'l' : logic=1; break;
		case '?' : print_help(stderr,1); break;
	}
	} while (next != -1 );
	/* Logic format is allowed only for GF(2) */
	if ((logic) && (field!=2)) {
		fprintf(stderr, "Logic output format is allowed only for GF(2) \n");
		exit(1);
	}
}

void
print_help(FILE * stream, int code)
{
	fprintf(stream, "Usage: eq-gen [options]\n");
	fprintf(stream, "\t-h: print this help\n");
	fprintf(stream, "\t-V: print version info\n");
	fprintf(stream, "\t-v: number of variables [default 6]\n");
	fprintf(stream, "\t-s: generated system will have at least one solution\n");
	fprintf(stream, "\t-f: field [default F_2]\n");
	fprintf(stream, "\t-p: probability [default 1/4]\n");
	fprintf(stream, "\t-n: number of equations [default 0] \n");
	fprintf(stream, "\t-d: degree of equations [default 2] \n");
	fprintf(stream, "\t-o: name of output file [default stdout] \n");
	fprintf(stream, "\t-l: logic fromat output [default off] - only for GF(2)\n");
	exit(code);
}
