/*
Program: cnf
File: cnf.pl
Author: Martin Voros
*/

/*
NOTES:
!!! works only for GF(2)
!!! be careful about empty line :-(
!!! Input has to be in logic format
*/

% operators deinition
:- op(300,fx,~).
:- op(400,yfx,*).
:- op(500,yfx,\/).
:- op(550,yfx,+).

tran(~(~X),Y) :-
	!,
	tran(X,Y).

tran(~(X * X),A) :-
	!,
	tran(~X, A).
tran(~(X * Y),A) :-
	!,
	tran((~X \/ ~Y), A).

tran(~(X \/ X),A) :-
	!,
	tran(~X,A).
tran(~(X \/ Y),A) :-
	!,
	tran((~X * ~Y),A).

tran(~(X + Y),A) :-
	!,
	tran((X * Y) \/ (~X * ~Y),  A).

tran((X * X),A) :-
	!,
	tran(X,A).

tran((X * Y),(A * B)) :-
	!,
	tran(X,A),
	tran(Y,B).

tran((X \/ X),A) :-
	!,
	tran(X,A).

tran((X \/ Y),(A \/ B)) :-
	!,
	tran(X,A),
	tran(Y,B).

tran((X + Y),(A \/ B) * (C \/ D)) :-
	!,
	tran(X,A),
	tran(Y,B),
	tran(~A,C),
	tran(~B,D).

tran(X,X).


dist((X * Y) \/ Z, (X \/ Z) * (Y \/ Z),true) :- !.
dist(X \/ (Y * Z), (X \/ Y) * (X \/ Z),true) :- !.
dist(X,X,fail).

cnf((X * X),A) :-
	!,
	cnf(X,A).

cnf((X * Y),(A * B)) :-
	!,
	cnf(X,A),
	cnf(Y,B).
cnf((X \/ Y),G) :-
	!,
	cnf(X,A),
	cnf(Y,B),
	dist((A \/ B),F,Flag),
	(Flag -> cnf(F,G) ; G = F).
cnf(X,X).

convertor(X, 0, Z):-
	tran(~X,F),
	cnf(F,T),
	flat(T,Z).
convertor(X, 1, Z):-
	tran(X,F),
	cnf(F,T),
	flat(T,Z).

split( X = Y, X , Y).

flat(X * Y, O):-
	!,
	flat(X,O),
	flat(Y,O).
flat(X,O):-
	dimacs([X],O).

flat2(X \/ Y, O):-
	!,
	flat2(X, O),
	flat2(Y, O).

flat2( ~X, O):-
	!,
	write(O, '-'),
	write(O, X),
	write(O, ' ').

flat2( X, O):-
	!,
	write(O, X),
	write(O, ' ').

dimacs([], _).
dimacs([X|Xs], O):-!,
	flat2(X,O),
	write(O, '0'),
	nl(O),
	dimacs(Xs, O).

% MAIN PROGRAM
lo(IN,OUT):-
	open(IN, read , S),
	open(OUT, write, O),
	 repeat,
               read(S,T),
		( 
		T == end_of_file,
		!;
		split(T, X , Y),
		convertor(X,Y,O),
		fail
		),
	close(O),
	close(S).
